"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var functions_1 = require("../functions");
var chai_1 = require("chai");
var spies = require("chai-spies");
chai_1.use(spies);
var consoleSpy = chai_1.spy.on(console, 'log');
describe('Function tests', function () {
    describe('Consumer', function () {
        it('sink does nothing, no side effects', function () {
            var value = 0;
            var sinkFunc = functions_1.Consumer.sink();
            sinkFunc(value);
            chai_1.expect(value).to.equal(0);
        });
        it('logger, logs the value to console', function () {
            var value = "THIS IS THE VALUE";
            var logFunc = functions_1.Consumer.logger();
            logFunc(value);
            chai_1.expect(consoleSpy).to.have.been.called.with(value);
        });
    });
    describe('Transformer', function () {
        it('identity function returns the value passed in', function () {
            var valueStruct = { i: 1, o: 'value' };
            var identity = functions_1.Transformer.identity();
            var returned = identity(valueStruct);
            chai_1.expect(returned).to.equal(valueStruct);
        });
        it('logger function returns the value passed in, and logs to console', function () {
            var valueStruct = { i: 1, o: 'LOG ME PLZ' };
            var logger = functions_1.Transformer.logger();
            var returned = logger(valueStruct);
            chai_1.expect(returned).to.equal(valueStruct);
            chai_1.expect(consoleSpy).to.have.been.called.with(valueStruct);
        });
    });
    describe('Comparator', function () {
        it('default comparator returns -1 if first value less than second', function () {
            var value1 = 1;
            var value2 = 2;
            var result = functions_1.Comparator.default()(value1, value2);
            chai_1.expect(result).lessThan(0);
        });
        it('default comparator returns 1 if first value greater than than second', function () {
            var value1 = 2;
            var value2 = 1;
            var result = functions_1.Comparator.default()(value1, value2);
            chai_1.expect(result).greaterThan(0);
        });
        it('default comparator returns -1 if first value less than second', function () {
            var value1 = 1;
            var value2 = 1;
            var result = functions_1.Comparator.default()(value1, value2);
            chai_1.expect(result).to.equal(0);
        });
    });
    describe('BiPredicate', function () {
        it('defaultEquality returns a function that returns true if two items === each other', function () {
            var value1 = 1;
            var value2 = 1;
            var result = functions_1.BiPredicate.defaultEquality()(value1, value2);
            chai_1.expect(result).to.eq(true);
        });
        it('defaultEquality returns a function that returns false if two items do not ===', function () {
            var value1 = 2;
            var value2 = 1;
            var result = functions_1.BiPredicate.defaultEquality()(value1, value2);
            chai_1.expect(result).to.eq(false);
        });
        it('defaultEquality returns a function that returns true if two items === each other', function () {
            var value1 = { i: '1' };
            var value2 = { i: '1' };
            var result = functions_1.BiPredicate.defaultEquality()(value1, value2);
            chai_1.expect(result).to.eq(false);
        });
        it('defaultEquality returns a function that returns true if two items === each other', function () {
            var value1 = { i: '1' };
            var value2 = value1;
            var result = functions_1.BiPredicate.defaultEquality()(value1, value2);
            chai_1.expect(result).to.eq(true);
        });
    });
});
